// lscp_server_test.c

#include "lscp_server.h"
#include "lscp_parser.h"

#define SERVER_PORT 3941

#if defined(WIN32)
static WSADATA _wsaData;
#endif

////////////////////////////////////////////////////////////////////////

lscp_status_t server_callback ( lscp_connect_t *pConnect, const char *pchBuffer, int cchBuffer, void *pvData )
{
    lscp_status_t ret = LSCP_OK;
    lscp_parser_t tok;
    const char   *pszResult;

    lscp_parser_init(&tok, pchBuffer, cchBuffer);

    fprintf(stderr, "server_callback: addr=%s port=%d: [%s ...]\n", inet_ntoa(pConnect->client.addr.sin_addr), htons(pConnect->client.addr.sin_port), tok.pchBuffer);

    if (lscp_parser_test(&tok, "GET")) {
        if (lscp_parser_test(&tok, "CHANNEL")) {
            if (lscp_parser_test(&tok, "INFO")) {
            }
            else if (lscp_parser_test(&tok, "VOICE_COUNT")) {
            }
            else if (lscp_parser_test(&tok, "STREAM_COUNT")) {
            }
            else if (lscp_parser_test(&tok, "BUFFER_FILL")) {
                if (lscp_parser_test(&tok, "BYTES")) {
                }
                else if (lscp_parser_test(&tok, "PERCENTAGE")) {
                }
            }
            else if (lscp_parser_test(&tok, "AUDIO_OUTPUT_TYPE")) {
            }
            else if (lscp_parser_test(&tok, "AUDIO_OUTPUT_CHANNEL")) {
            }
            else ret = LSCP_FAILED;
        }
        else if (lscp_parser_test(&tok, "CHANNELS")) {
        }
        else if (lscp_parser_test(&tok, "AVAILABLE_ENGINES")) {
        }
        else if (lscp_parser_test2(&tok, "ENGINE", "INFO")) {
        }
        else ret = LSCP_FAILED;
    }
    else if (lscp_parser_test(&tok, "SET")) {
        if (lscp_parser_test(&tok, "CHANNEL")) {
            if (lscp_parser_test(&tok, "VOLUME")) {
            }
            else if (lscp_parser_test(&tok, "AUDIO_OUTPUT_CHANNEL")) {
            }
            else if (lscp_parser_test(&tok, "MIDI_INPUT_CHANNEL")) {
            }
            else if (lscp_parser_test(&tok, "MIDI_INPUT_PORT")) {
            }
            else ret = LSCP_FAILED;
        }
        else ret = LSCP_FAILED;
    }
    else if (lscp_parser_test(&tok, "LOAD")) {
        if (lscp_parser_test(&tok, "ENGINE")) {
        }
        else if (lscp_parser_test(&tok, "INSTRUMENT")) {
        }
        else ret = LSCP_FAILED;
    }
    else if (lscp_parser_test2(&tok, "ADD", "CHANNEL")) {
    }
    else if (lscp_parser_test2(&tok, "REMOVE", "CHANNEL")) {
    }
    else if (lscp_parser_test2(&tok, "SUBSCRIBE", "NOTIFICATION")) {
        ret = lscp_server_subscribe(pConnect, lscp_parser_nextint(&tok));
    }
    else if (lscp_parser_test2(&tok, "UNSUBSCRIBE", "NOTIFICATION")) {
        ret = lscp_server_unsubscribe(pConnect);
    }
    else ret = LSCP_FAILED;

    lscp_parser_free(&tok);

    pszResult = (ret == LSCP_OK ? "OK" : "ERR:1:Oops");

    return lscp_server_result(pConnect, pszResult, strlen(pszResult));
}

////////////////////////////////////////////////////////////////////////

int main (int argc, char argv[] )
{
    lscp_server_t *pServer;
    char szLine[80];

#if defined(WIN32)
    if (WSAStartup(MAKEWORD(1, 1), &_wsaData)) {
        fprintf(stderr, "lscp_server: WSAStartup failed.\n");
        return -1;
    }
#endif

    pServer = lscp_server_create(SERVER_PORT, server_callback, NULL);
    if (pServer == NULL)
        return -1;

    while (fgets(szLine, sizeof(szLine), stdin)) {
        if (strncmp(szLine, "list", 4) == 0) {
            lscp_connect_t *p = pServer->connects.first;
            while (p) {
                printf("client: sock=%d addr=%s port=%d udp:port=%d ping=%d.\n", p->client.sock, inet_ntoa(p->client.addr.sin_addr), ntohs(p->client.addr.sin_port), p->port, p->ping);
                p = p->next;
            }
        }
        else
        if (strncmp(szLine, "exit", 4) == 0)
            break;
    }

    lscp_server_destroy(pServer);

#if defined(WIN32)
    WSACleanup();
#endif

    return 0;
}

// end of lscp_server_test.c

